/*
 *  setup.i
 *
 *  $Id: setup.i,v 1.6.14.1 2006/02/01 12:28:54 source Exp $
 *
 *  Extract the _file and _field information into a export file
 *  which can be used with the TABLEVIEW approach for 
 *  SQLTables and SQLColumns.
 *
 *  (C)Copyright 1993, 1994, 1995 OpenLink Software.
 *  All Rights Reserved.
 *
 *  The copyright above and this notice must be preserved in all
 *  copies of this source code.  The copyright above does not
 *  evidence any actual or intended publication of this source code.
 *
 *  This is unpublished proprietary trade secret of OpenLink Software.
 *  This source code may not be copied, disclosed, distributed, demonstrated
 *  or licensed except as authorized by OpenLink Software.
 *
 *  Parameters:
 *    {&ldb}	= logical database name
 */

pause 0 before-hide.

/*
 * Local variables
 */
define variable v_i 		as integer no-undo.
define variable v_j 		as integer no-undo.
define variable v_length	as integer no-undo.
define variable v_precision	as integer no-undo.
define variable v_max_length    as integer no-undo.
define variable v_scale		as integer no-undo.
define variable v_desc		as character format "X(255)" no-undo.


/*
 *  Maximum column length for char fields
 */
v_max_length = 4096.


/*
 *  L = Logical Database name
 */
put unformatted "L|" caps("{&ldb}") skip.

/*
 *  Now get all the table information 
 */
for each {&ldb}._file no-lock by _file-name:
    /*
     *  T = Normal Table
     *  S = System Table (all hidden tables are expected to be system tables)
     */
    if _hidden then put unformatted "S|". else put unformatted "T|".
    put unformatted _file-name "|".

    /*
     * Convert multiline descriptions into a single line
     * by replacing the newline character with a space
     */
    v_desc = trim(_desc).
    do while true:
        v_j = r-index(v_desc, "~012").
	if v_j = 0 or v_j = ? then leave.
	substring(v_desc, v_j, 1) = " ".
    end.
    do while true:
        v_j = r-index(v_desc, "~015").
	if v_j = 0 or v_j = ? then leave.
	substring(v_desc, v_j, 1) = " ".
    end.
    do while true:
       v_j = r-index (v_desc, "  ").
       if v_j = 0 or v_j = ? then leave.
       v_desc = substring(v_desc, 1, v_j) + trim(substring(v_desc, v_j + 1)).
    end.
    put unformatted trim(substring(v_desc, 1, 254)) skip.

    for each {&ldb}._field of {&ldb}._file no-lock by _field-name:

	/*
	 *  Calculate the length, scale and precision for this datatype
	 */
        assign
	  v_length	= length(string("", _field._format))
	  v_scale	= _field._decimals
	  v_precision	= 0.
	  .
	if _Dtype = 4 or _Dtype = 5 or _Dtype = 7 then do:
	   do v_i = 1 to length(_field._format):
	       if index(".", substring(_field._format, v_i, 1)) > 0 then leave.
	       if index("<>9Z", substring(_field._format, v_i, 1)) > 0 then
	          v_precision = v_precision + 1.
	   end.
	end.

	/*
	 * Sanity check
	 */
	if v_length = ? or v_length > v_max_length then v_length = v_max_length.
	if v_scale  = ? then v_scale  = 0.
	assign v_precision = v_precision + v_scale.
	if v_precision > v_max_length then v_precision = v_max_length.

        /*
	 * F = Field
	 */
	do v_i = 0 to _extent:
	  /*
	   *  At this time we skip arrays in hidden (=SYSTEM) tables
	   */
	  if ((v_i = 0 or _hidden) and _extent > 0) then next.
	  put unformatted "F|" _field-name.
	  if v_i > 0 then put unformatted "@" v_i.
	  put unformatted
	    "|" _Dtype "|" v_length "|" v_precision "|" v_scale "|" 
	    not _Mandatory "|".

	  /*
	   * Convert multiline descriptions into a single line
	   * by replacing the newline character with a space
	   */
	  v_desc = trim(_Field._desc).
	  do while true:
	      v_j = r-index(v_desc, "~012").
	      if v_j = 0 or v_j = ? then leave.
	      substring(v_desc, v_j, 1) = " ".
	  end.
	  do while true:
	      v_j = r-index(v_desc, "~015").
	      if v_j = 0 or v_j = ? then leave.
	      substring(v_desc, v_j, 1) = " ".
	  end.
	  do while true:
	     v_j = r-index (v_desc, "  ").
	     if v_j = 0 or v_j = ? then leave.
	     v_desc = substring(v_desc, 1, v_j) + trim(substring(v_desc, v_j + 1)).
	  end.
	  put unformatted trim(substring(v_desc, 1, 254)).
	  put unformatted "|" _Initial "|" _Order skip.
	end.
    end.
    put unformatted "." skip.
end.

/*
 *  Now get all the view information 
 */
for each {&ldb}._view no-lock by _view-name:
    /*
     *  V = View
     */
    put unformatted "V|" _view-name "|"  skip.
    for each {&ldb}._view-col of {&ldb}._view no-lock by _view-col._col-name:
        find first {&ldb}._view-ref of _view where
	  _view-col._base-col = 
	  	_view-ref._ref-table + "." + _view-ref._base-col 
	  or _view-col._base-col begins 
	  	"(" + _view-ref._ref-table + "." + _view-ref._base-col 
	  no-lock no-error.
	if not available {&ldb}._view-ref then next.
	find {&ldb}._file where 
	  _file-name = {&ldb}._view-ref._ref-table no-lock.
	find {&ldb}._field of {&ldb}._file where
	  _field-name = {&ldb}._view-ref._base-col no-lock.

	/*
	 *  Calculate the length, scale and precision for this datatype
	 */
        assign
	  v_length	= length(string("", _field._format))
	  v_scale	= _field._decimals
	  v_precision	= 0.
	  .
	if _Dtype = 4 or _Dtype = 5 or _Dtype = 7 then do:
	   do v_i = 1 to length(_field._format):
	       if index(".", substring(_field._format, v_i, 1)) > 0 then leave.
	       if index("<>9Z", substring(_field._format, v_i, 1)) > 0 then
	          v_precision = v_precision + 1.
	   end.
	end.

	/*
	 * Sanity check
	 */
	if v_length = ? or v_length > v_max_length then v_length = v_max_length.
	if v_scale  = ? then v_scale  = 0.
	assign v_precision = v_precision + v_scale.
	if v_precision > v_max_length then v_precision = v_max_length.

        /*
	 * F = Field
	 */
	do v_i = 0 to _extent:
	  /*
	   *  At this time we skip arrays in hidden (=SYSTEM) tables
	   */
	  if ((v_i = 0 or _hidden) and _extent > 0) then next.
	  put unformatted "F|" _view-col._col-name.
	  if v_i > 0 then put unformatted "@" v_i.
	  put unformatted
	    "|" _Dtype "|" v_length "|" v_precision "|" v_scale "|" 
	    not _Mandatory "|".
	  /*
	   * Convert multiline descriptions into a single line
	   * by replacing the newline character with a space
	   */
	  v_desc = trim(_Field._desc).
	  do while true:
	      v_j = r-index(v_desc, "~012").
	      if v_j = 0 or v_j = ? then leave.
	      substring(v_desc, v_j, 1) = " ".
	  end.
	  do while true:
	      v_j = r-index(v_desc, "~015").
	      if v_j = 0 or v_j = ? then leave.
	      substring(v_desc, v_j, 1) = " ".
	  end.
	  do while true:
	     v_j = r-index (v_desc, "  ").
	     if v_j = 0 or v_j = ? then leave.
	     v_desc = substring(v_desc, 1, v_j) + trim(substring(v_desc, v_j + 1)).
	  end.
	  put unformatted trim(substring(v_desc, 1, 254)).
	  put unformatted "|" _Initial "|" _Order skip.
	end.
    end.
    put unformatted "." skip.
end.

/*
 *  Signal end of set
 */
put unformatted "." skip.
