/*
 *  setup.p
 *
 *  $Id: setup.p,v 1.6 1998/02/12 09:00:57 source Exp $
 *
 *  Create a .dat and .pf file for the Progress Database Agent
 *
 *  (C)Copyright 1993-1997 OpenLink Software.
 *  All Rights Reserved.
 *
 *  The copyright above and this notice must be preserved in all
 *  copies of this source code.  The copyright above does not
 *  evidence any actual or intended publication of this source code.
 *
 *  This is unpublished proprietary trade secret of OpenLink Software.
 *  This source code may not be copied, disclosed, distributed, demonstrated
 *  or licensed except as authorized by OpenLink Software.
 */

define variable v_i       as integer   format "Z9"    no-undo.
define variable v_dbname  as character format "X(70)" no-undo.
define variable v_connect as character format "X(70)" no-undo.
define variable v_conn_ar as character format "X(30)" no-undo extent 11.
define variable v_conn_ok as logical   format "OK/"   no-undo.
define variable v_oplpath as character format "X(40)" no-undo.
define variable v_filenm  as character format "X(40)" no-undo.
define variable v_tblfile as character format "X(40)" no-undo.
define variable v_pffile  as character format "X(40)" no-undo.


/*
 *  Used forms
 */

form
   skip(1)
   " Enter all the Distibuted Progress Databases you want to connect to in one "
   skip
   " session. The first database will be the master database and upto 10 "
   skip
   " different databases can be connected to at same time. For each database you "
   skip
   " will have to provide the full path to the database e.g. /dbs/dlc73C/demo.db "
   skip
   " plus the parameters Progress needs to connect to this database e.g.: " 
   skip(1)
   "  - multi user connection  : no connect parameters needed " skip
   "                             for shared memory connection " skip
   "  - distributed connection : -S Service -H Host -N TCP" skip
   "  - single user connection :  -1" skip
   "  - readonly connection    :  -1 -RO" skip
   skip(1)
   " The program will try and make a connection to these databases to check the"
   skip
   " information you entered, prior to extracting the actual information. "
   skip
   " Please make sure all databases are online prior to running this script "
   skip
   " otherwise this check will fail and you will not be allowed to continue. " 
   skip
   with frame f_header title " HELP ".

form 
   v_i label "#"
   v_dbname at 4 label "Database Name" 
        Help "The full pathname of the database you want to connect to"
   v_conn_ok label "OK"
   skip
   v_connect at 4 label "Connect Parameters"
        Help "The connect parameters e.g. -1, -RO or -S SERV -H HOST -N TCP"
   with frame f_connect title " ENTER CONNECT INFORMATION (PRESS F4 TO END) " 5 down.

form
  skip (1)
  " The setup program is about to create two files. One contains the connection "
  skip
  " information (.pf) and the other contains table and column information (.dat)."
  skip(1)
  " Please enter the directory in which the files will be generated and the "
  skip
  " filename (without extention): "
  skip(1)
  "     Directory : " v_oplpath no-label
  skip
  "     Filename  : " v_filenm format "X(15)" no-label
  skip(1)
  " Make sure the directory exists and you have permission to create files in "
  skip
  " there, otherwise the program will abort. "
  with frame f_generate title " ENTER DIRECTORY INFORMATION ".

form 
    skip(1)
    " The program has generated two files for you which contain information on "
    skip
    " how to connect to these databases and table/column information for the "
    skip
    " catalog calls. You will need to modify the following parameters in your "
    skip
    " 'Session Rules' Book (oplrqb.ini) in order for the Progress agent to "
    skip
    " use these values (example using Progress version 7.3C): "
    skip(1)
    "     [Environment PROGRESS73C] "
    skip
    "     TABLEVIEW = "  v_tblfile
    skip(1)
    "     [generic_pro73c]"
    skip
    "     Database = -pf "   v_pffile
    skip(1)
    " Make sure you are using full pathnames for both files and that users have  "
    skip
    " access to these directories and files, otherwise they will not be able to "
    skip
    " connect ."
   with frame f_footer title " NOTICE ".

/* 
 *  Disconnect from all databases before continuing
 */
do while true:
    if ldbname(1) = ? then leave.
    message " Disconnecting from " + pdbname(1).
    disconnect value(ldbname(1)).
end.
hide message no-pause.


/*
 *  Display the help screen so people know what to expect
 */
display with frame f_header.
pause message " Press any key to start ".
hide message no-pause.


/*
 *  Now ask for database names and connect parameters untill the 
 *  User presses the F4 function key
 */
v_i = 1.
pause 0 before-hide.
repeat on endkey undo, leave on error undo, retry with frame f_connect:
  if retry then do:
    hide message no-pause.
    message " Error in connect string or database not available ".
    pause 3 no-message.
    hide message no-pause.
  end.

  display v_i.
  prompt-for v_dbname v_connect.
  assign v_dbname v_connect.

  connect value (v_connect + " " + v_dbname).

  assign
      v_conn_ar[v_i] = v_connect
      v_conn_ok = connected(ldbname(v_i)).
  display v_conn_ok.

  v_i = v_i + 1.
  if v_i = 11 then leave.
  down with frame f_connect.
end.

if ldbname(1) = ? then do:
    message " No tables selected ".
    return.
end.


/*
 *  Now ask for the name to use for the generated files
 */
v_oplpath = "/user/openlink/bin".
v_filenm = "tableview".
do on endkey undo, retry on error undo, retry:
  if retry then do:
    message " File creation failed. Check directory or permissions. ".
    pause 10 no-message.
    quit.
  end.

  do with frame f_generate:
    display v_oplpath v_filenm.
    update v_oplpath v_filenm.
  end.

  v_i = 1.
  message " Creating .pf file to use ".
  output to value(v_oplpath + "/" + v_filenm + ".pf").
  do while true:
      if pdbname(v_i) = ? then leave.
      put unformatted 
	  "# "					skip
	  "# Connect to " ldbname(v_i)		skip
	  "# " 					skip
	  "-db " pdbname(v_i)			skip
	  "-ld " ldbname(v_i)			skip
	  v_conn_ar[v_i]				skip
	  .
      v_i = v_i + 1.
  end.
  output close.

  message " Creating .dat file with table information ".
  v_i = 1.
  output to value (v_oplpath + "/" + v_filenm + ".dat").
  output close.
  do while true:
    if ldbname(v_i) = ? then leave.
    output to s_temp.p.

    put unformatted 
	"output to " v_oplpath "/" v_filenm ".dat append."	skip
	"~{setup.i &ldb=" ldbname(v_i) "~}"			skip
	.

    output close.
    compile s_temp.p.
    run s_temp.p.
    v_i = v_i + 1.
  end.
  output close.
end.

/*
 *  Display the help screen so people know what to do next
 */
hide message no-pause.
clear no-pause.
display 
  v_oplpath + "/" + v_filenm + ".dat" @ v_tblfile  no-label
  v_oplpath + "/" + v_filenm + ".pf"  @ v_pffile no-label
  with frame f_footer.

pause message " Press any key to close this program ".
quit.
